package com.goldencode.p2j.ui.client.chui.driver.web;

import java.nio.*;
import java.util.Deque;
import java.util.concurrent.*;

import org.eclipse.jetty.websocket.api.*;

public class PushMessagesWorker 
extends Thread
{

   /** Keyboard input queue */
   private Deque<Object> messages = new ConcurrentLinkedDeque<>();
   
   /** Remote endpoint */
   private volatile RemoteEndpoint remote;  
   
   /** Running flag */
   private boolean running = true;
   
   /**
    * 
    * @param remote
    */
   public PushMessagesWorker(RemoteEndpoint remote)
   {
      this.remote = remote;
      this.start();
   }
   
   /**
    * 
    * @param message
    */
   public void pushMessage(Object message)
   {
      messages.offerLast(message);
      // notify
      synchronized (messages)
      {
         messages.notify();
      }
   }
   
   /**
    * 
    */
   @Override
   public void run()
   {
      while(running)
      {
         if (remote != null)
         {
            try
            {
               // are messages in the queue ?
               if (messages.isEmpty())
               {
                  // no messages on the queue. wait.
                  synchronized (messages)
                  {
                     messages.wait();
                  }                  
               }
               // get a message from queue
               Object message = messages.pollFirst();
               // test message type
               if (message instanceof ByteBuffer)
               {
                  remote.sendBytes((ByteBuffer) message);
               }
               else if (message instanceof String)
               {
                  remote.sendString((String) message);
               }
            }
            catch (Exception e)
            {
               e.printStackTrace();
            }
         }
      }
   }

   /**
    * 
    */
   public void stopPushing()
   {
      running = false;
      this.interrupt();
      try
      {
         this.join(5000);
      }
      catch (InterruptedException e)
      {
         e.printStackTrace();
      }
   }   
}
